@@ -1,21 +1,19 @@
 (*
  * ocamlweb - A WEB-like tool for ocaml
- * Copyright (C) 1999-2001 Jean-Christophe FILLITRE and Claude MARCH
- * 
+ * Copyright (C) 1999-2001 Jean-Christophe FILLIÂTRE and Claude MARCHÉ
+ *
  * This software is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
  * License version 2, as published by the Free Software Foundation.
- * 
+ *
  * This software is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * 
+ *
  * See the GNU Library General Public License version 2 for more details
  * (enclosed in the file LGPL).
  *)
 
-(*i $Id$ i*)
-
 (*i*)
 open Filename
 open Printf
@@ -30,10 +28,11 @@ open Doclexer
 let usage () =
   prerr_endline "";
   prerr_endline "Usage: ocamlweb <options and files>";
-  prerr_endline "  -o <file>      write output in file <file>";
-  prerr_endline "  --dvi          output the DVI";
-  prerr_endline "  --ps           output the PostScript";
-  prerr_endline "  --html         output the HTML";
+  prerr_endline "  -o <file>      write output into <file>";
+  prerr_endline "  --dvi          output in DVI format (using latex)";
+  prerr_endline "  --ps           output in PostScript format (using dvips)";
+  prerr_endline "  --pdf          output in PDF format (using pdflatex)";
+  prerr_endline "  --html         output in HTML format (using hevea)";
   prerr_endline "  --hevea-option <opt>";
   prerr_endline "                 pass an option to hevea (HTML output)";
   prerr_endline "  -s             (short) no titles for files";
@@ -48,19 +47,20 @@ let usage () =
   prerr_endline "  --latex-option <opt>";
   prerr_endline "                 pass an option to the LaTeX package ocamlweb.sty";
   prerr_endline "  --class-options <opt>";
-  prerr_endline "                 set the document class options (defaults to `12pt')";
-  prerr_endline "  --old-fullpage uses LaTeX package fullpage with no option";
+  prerr_endline "                 set the document class options (default is `12pt')";
+  prerr_endline "  --encoding <e> set the character encoding (default is 'utf8')";
+  prerr_endline "  --old-fullpage use LaTeX package fullpage with no option";
   prerr_endline "  -p <string>    insert something in LaTeX preamble";
   prerr_endline "  --files <file> read file names to process in <file>";
   prerr_endline "  --quiet        quiet mode";
   prerr_endline "  --no-greek     disable use of greek letters for type variables";
   prerr_endline "";
-  prerr_endline 
+  prerr_endline
     "On-line documentation at http://www.lri.fr/~filliatr/ocamlweb/\n";
   exit 1
 
 
-(*s \textbf{License informations.} Printed when using the option 
+(*s \textbf{License informations.} Printed when using option
     \verb!--warranty!. *)
 
 let copying () =
@@ -80,18 +80,18 @@ See the GNU Library General Public License version 2 for more details
 
 (*s \textbf{Banner.} Always printed. Notice that it is printed on error
     output, so that when the output of \ocamlweb\ is redirected this header
-    is not (unless both standard and error outputs are redirected, of 
+    is not (unless both standard and error outputs are redirected, of
     course). *)
 
 let banner () =
   eprintf "This is ocamlweb version %s, compiled on %s\n"
     Version.version Version.date;
-  eprintf 
-    "Copyright (c) 1999-2000 Jean-Christophe Fillitre and Claude March\n";
+  eprintf
+    "Copyright (c) 1999-2000 Jean-Christophe Filliâtre and Claude Marché\n";
   eprintf
   "This is free software with ABSOLUTELY NO WARRANTY (use option -warranty)\n";
   flush stderr
-    
+
 
 (*s \textbf{Separation of files.} Files given on the command line are
     separated according to their type, which is determined by their suffix.
@@ -110,9 +110,9 @@ let what_file f =
     File_impl (make_caml_file f)
   else if check_suffix f ".mli" then
     File_intf (make_caml_file f)
-  else if check_suffix f ".mll" then 
+  else if check_suffix f ".mll" then
     File_lex (make_caml_file f)
-  else if check_suffix f ".mly" then 
+  else if check_suffix f ".mly" then
     File_yacc (make_caml_file f)
   else if check_suffix f ".tex" then
     File_other f
@@ -121,7 +121,7 @@ let what_file f =
     exit 1
   end
 
-(*s \textbf{Reading file names from a file.} 
+(*s \textbf{Reading file names from a file.}
     File names may be given
     in a file instead of being given on the command
     line. [(files_from_file f)] returns the list of file names contained
@@ -139,7 +139,7 @@ let files_from_file f =
 	  | ' ' | '\t' | '\n' ->
 	      if Buffer.length buf > 0 then l := (Buffer.contents buf) :: !l;
 	      Buffer.clear buf
-	  | c -> 
+	  | c ->
 	      Buffer.add_char buf c
       done; []
     with End_of_file ->
@@ -161,6 +161,7 @@ let files_from_file f =
 let output_file = ref ""
 let dvi = ref false
 let ps = ref false
+let pdf = ref false
 let html = ref false
 let hevea_options = ref ([] : string list)
 
@@ -186,7 +187,7 @@ let parse () =
 	index := false; parse_rec rem
     | ("-o" | "--output") :: f :: rem ->
 	output_file := f; parse_rec rem
-    | ("-o" | "--output") :: [] -> 
+    | ("-o" | "--output") :: [] ->
 	usage ()
     | ("-s" | "--short") :: rem ->
 	short := true; parse_rec rem
@@ -194,11 +195,13 @@ let parse () =
 	dvi := true; parse_rec rem
     | ("-ps" | "--ps") :: rem ->
 	ps := true; parse_rec rem
+    | ("-pdf" | "--pdf") :: rem ->
+	pdf := true; parse_rec rem
     | ("-html" | "--html") :: rem ->
 	html := true; parse_rec rem
-    | ("-hevea-option" | "--hevea-option") :: [] -> 
+    | ("-hevea-option" | "--hevea-option") :: [] ->
 	usage ()
-    | ("-hevea-option" | "--hevea-option") :: s :: rem -> 
+    | ("-hevea-option" | "--hevea-option") :: s :: rem ->
 	hevea_options := s :: !hevea_options; parse_rec rem
     | ("-extern-defs" | "--extern-defs") :: rem ->
 	extern_defs := true; parse_rec rem
@@ -223,12 +226,16 @@ let parse () =
 	add_latex_option s; parse_rec rem
     | "--latex-option" :: [] ->
 	usage ()
+    | "--encoding" :: s :: rem ->
+	encoding := s; parse_rec rem
+    | "--encoding" :: [] ->
+	usage ()
     | "--old-fullpage" :: rem ->
 	fullpage_headings := false; parse_rec rem
 
-    | ("-impl" | "--impl") :: f :: rem -> 
+    | ("-impl" | "--impl") :: f :: rem ->
 	check_if_file_exists f;
-	let n = 
+	let n =
 	  if Filename.check_suffix f ".mll" || Filename.check_suffix f ".mly"
           then Filename.chop_extension f else f
 	in
@@ -242,26 +249,27 @@ let parse () =
 	add_file i; parse_rec rem
     | ("-intf" | "--intf") :: [] ->
 	usage ()
-    | ("-tex" | "--tex") :: f :: rem -> 
+    | ("-tex" | "--tex") :: f :: rem ->
 	add_file (File_other f); parse_rec rem
     | ("-tex" | "--tex") :: [] ->
 	usage ()
     | ("-files" | "--files") :: f :: rem ->
-	List.iter (fun f -> add_file (what_file f)) (files_from_file f); 
+	List.iter (fun f -> add_file (what_file f)) (files_from_file f);
 	parse_rec rem
     | ("-files" | "--files") :: [] ->
 	usage ()
-    | f :: rem -> 
+    | f :: rem ->
 	add_file (what_file f); parse_rec rem
-  in 
+  in
   parse_rec (List.tl (Array.to_list Sys.argv));
   List.rev !files
 
 (*s The following function produces the output. The default output is
-    the \LaTeX\ document: in that case, we just call [Web.produce_document]. 
-    If option \verb!-dvi!, \verb!-ps! or \verb!-html! is invoked, then
-    we make calls to \verb!latex!, \verb!dvips! and/or \verb!hevea!
-    accordingly. *)
+  the \LaTeX\ document: in that case, we just call
+  [Web.produce_document].  If option \verb!-dvi!, \verb!-ps!,
+  \verb!-pdf!, or \verb!-html! is invoked, then we make calls to
+  \verb!latex!, \verb!dvips!, \verb!pdflatex!, and/or \verb!hevea!
+  accordingly. *)
 
 let locally dir f x =
   let cwd = Sys.getcwd () in
@@ -290,7 +298,7 @@ let cat file =
     close_in c
 
 let copy src dst =
-  let cin = open_in src 
+  let cin = open_in src
   and cout = open_out dst in
   try
     while true do Pervasives.output_char cout (input_char cin) done
@@ -298,7 +306,7 @@ let copy src dst =
     close_in cin; close_out cout
 
 let produce_output fl =
-  if not (!dvi || !ps || !html) then begin
+  if not (!dvi || !ps || !pdf || !html) then begin
     if !output_file <> "" then set_output_to_file !output_file;
     produce_document fl
   end else begin
@@ -306,54 +314,63 @@ let produce_output fl =
     let basefile = chop_suffix texfile ".tex" in
     set_output_to_file texfile;
     produce_document fl;
-    let command = 
+    let latex = if !pdf then "pdflatex" else "latex" in
+    let command =
       let file = basename texfile in
-      let file = 
-	if !quiet then sprintf "'\\nonstopmode\\input{%s}'" file else file 
+      let file =
+	if !quiet then sprintf "'\\nonstopmode\\input{%s}'" file else file
       in
-      sprintf "(latex %s && latex %s) 1>&2 %s" file file
+      sprintf "(%s %s && %s %s) 1>&2 %s" latex file latex file
 	(if !quiet then "> /dev/null" else "")
     in
     let res = locally (dirname texfile) Sys.command command in
     if res <> 0 then begin
-      eprintf "Couldn't run LaTeX successfully\n"; 
+      eprintf "Couldn't run LaTeX successfully\n";
       clean_and_exit basefile res
     end;
     let dvifile = basefile ^ ".dvi" in
     if !dvi then begin
-      if !output_file <> "" then 
+      if !output_file <> "" then
 	(* we cannot use Sys.rename accross file systems *)
-	copy dvifile !output_file 
-      else 
+	copy dvifile !output_file
+      else
 	cat dvifile
     end;
     if !ps then begin
-      let psfile = 
-	if !output_file <> "" then !output_file else basefile ^ ".ps" 
+      let psfile =
+	if !output_file <> "" then !output_file else basefile ^ ".ps"
       in
-      let command = 
-	sprintf "dvips %s -o %s %s" dvifile psfile 
+      let command =
+	sprintf "dvips %s -o %s %s" dvifile psfile
 	  (if !quiet then "> /dev/null 2>&1" else "")
       in
       let res = Sys.command command in
       if res <> 0 then begin
-	eprintf "Couldn't run dvips successfully\n"; 
+	eprintf "Couldn't run dvips successfully\n";
 	clean_and_exit basefile res
       end;
       if !output_file = "" then cat psfile
     end;
+    if !pdf then begin
+      let pdffile = basefile ^ ".pdf" in
+      if !output_file <> "" then
+	(* we cannot use Sys.rename accross file systems *)
+	copy pdffile !output_file
+      else
+	cat pdffile
+    end;
     if !html then begin
-      let htmlfile = 
-	if !output_file <> "" then !output_file else basefile ^ ".html" 
+      let htmlfile =
+	if !output_file <> "" then !output_file else basefile ^ ".html"
       in
       let options = String.concat " " (List.rev !hevea_options) in
-      let command = 
+      let command =
 	sprintf "hevea %s ocamlweb.sty %s -o %s %s" options texfile htmlfile
 	  (if !quiet then "> /dev/null 2>&1" else "")
       in
       let res = Sys.command command in
       if res <> 0 then begin
-	eprintf "Couldn't run hevea successfully\n"; 
+	eprintf "Couldn't run hevea successfully\n";
 	clean_and_exit basefile res
       end;
       if !output_file = "" then cat htmlfile
@@ -370,11 +387,11 @@ let main () =
     let l = List.map read_one_file files in
     if !web_style then begin
       if not !web && not !quiet then begin
-	eprintf 
+	eprintf
 	  "Warning: web sections encountered while in noweb style, ignored.\n";
-	flush stderr    
+	flush stderr
       end
-    end else 
+    end else
       web := false;
     if not !web then add_latex_option "noweb";
     produce_output l
